{% extends "!layout.html" %}

{# Taken and modified from sphinx-bootstrap-theme demo site
   (https://github.com/ryan-roemer/sphinx-bootstrap-theme/blob/master/demo/source/_templates/layout.html). #}

{# Add github banner (from: https://github.com/blog/273-github-ribbons). #}
{% block header %}
  {{ super() }}
  <a href="https://github.com/biocore/emperor"
     class="visible-desktop hidden-xs"><img
    id="gh-banner"
    style="position: absolute; top: 50px; right: 0; border: 0;"
    src="https://s3.amazonaws.com/github/ribbons/forkme_right_orange_ff7600.png"
    alt="Fork me on GitHub"></a>
  <script>
    // Adjust banner height.
    $(function () {
      var navHeight = $(".navbar .container").css("height");
      $("#gh-banner").css("top", navHeight);
    });
  </script>
{% endblock %}

{% block footer %}
  {{ super() }}
{% endblock %}
